package com.jsf.conventions.producer;

import com.jsf.conventions.event.BundleChangeEvent;
import com.jsf.conventions.qualifier.BundleSulfix;
import com.jsf.conventions.qualifier.IgnoreBundleSulfix;
import com.jsf.conventions.qualifier.Name;
import com.jsf.conventions.util.ResourceBundle;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;
import javax.enterprise.context.ApplicationScoped;
import javax.enterprise.event.Observes;
import javax.enterprise.inject.Default;
import javax.enterprise.inject.Produces;
import javax.enterprise.inject.spi.InjectionPoint;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;

/**
 *
 * @author rmpestano
 */
@Named
@ApplicationScoped
public class ResourceBundleProducer implements Serializable {

    private static final long serialVersionUID = 1L;
    private static final String DEFAULT_BASE_NAME = "/MessageResource";
    @Inject @BundleSulfix
    protected String bundleSulfix;
    private String baseName;
    private Map<String, ResourceBundle> bundleMap = new HashMap<String, ResourceBundle>();
    private Logger log = Logger.getLogger(getClass().getSimpleName());
    

    public ResourceBundleProducer() {
        String baseNameParam = (String) FacesContext.getCurrentInstance().getExternalContext().getInitParameterMap().get("BUNDLE_BASE_NAME");
        if(baseNameParam == null){
            baseName = DEFAULT_BASE_NAME;
        }
        else{
            baseName = baseNameParam;
        }
    }


    /**
     * This method is the factory default for ResourceBundle. It creates the
     * ResourceBundle based on a file called messages.properties.
     */
    @Produces
    @Default
    public ResourceBundle create(InjectionPoint ip) {

        if (ip != null && ip.getAnnotated().isAnnotationPresent(Name.class)) {
            baseName = ip.getAnnotated().getAnnotation(Name.class).value();
        } else {
            String baseNameParam = (String) FacesContext.getCurrentInstance().getExternalContext().getInitParameterMap().get("BUNDLE_BASE_NAME");
            if (baseNameParam == null) {
                baseName = DEFAULT_BASE_NAME;
            } else {
                baseName = baseNameParam;
            }
        }
        if(ip != null && ip.getAnnotated().isAnnotationPresent(IgnoreBundleSulfix.class)){
           return  getResourceBundleWithoutSulfix(baseName);
        }

        return getResourceBundle(baseName);
    }

     
    public ResourceBundle getResourceBundle(String baseName) {
        ResourceBundle bundle = null;
        if (bundleMap.containsKey(baseName + "_" + bundleSulfix)) {
            bundle = bundleMap.get(baseName + "_" + bundleSulfix);

        } else {
            try {
                bundle = new ResourceBundle(getClass().getResourceAsStream(baseName + "_" + bundleSulfix + ".properties"));
            } catch (Exception e) {
                log.warning("CONVENTIONS:Problems trying to find resource bundle:"+baseName + "_" + bundleSulfix + ".properties");
            }
            bundleMap.put(baseName + "_" + bundleSulfix, bundle);
        }

        return bundle;
    }
    
      public ResourceBundle getResourceBundleWithoutSulfix(String baseName) {
        ResourceBundle bundle = null;
        if (bundleMap.containsKey(baseName)) {
            bundle = bundleMap.get(baseName);

        } else {
            try {
                bundle = new ResourceBundle(getClass().getResourceAsStream(baseName + ".properties"));
            } catch (Exception e) {
                log.warning("CONVENTIONS:Problems trying to find resource bundle:"+baseName + ".properties");
            }
            bundleMap.put(baseName, bundle);
        }

        return bundle;
    }


    public Map<String, ResourceBundle> getBundleMap() {
        return bundleMap;
    }

    public void setBundleMap(Map<String, ResourceBundle> map) {
        this.bundleMap = map;
    }

    public String getBaseName() {
        return baseName;
    }

    public void setBaseName(String baseName) {
        this.baseName = baseName;
    }
    
    public void onBundleSulfixChange(@Observes BundleChangeEvent event){
        this.bundleSulfix = event.getBundleSulfix();
    }
}
